@@ -29,7 +29,7 @@ from .parameterized import (
     descendents, get_logger, instance_descriptor, basestring)
 
 from .parameterized import (batch_watch, depends, output, # noqa: api import
-                            discard_events, edit_constant)
+                            discard_events, edit_constant, instance_descriptor)
 from .parameterized import logging_level     # noqa: api import
 from .parameterized import shared_parameters # noqa: api import
 
@@ -41,7 +41,7 @@ from numbers import Real
 # only two required files.
 try:
     from .version import Version
-    __version__ = str(Version(fpath=__file__, archive_commit="9123ba0e", reponame="param"))
+    __version__ = str(Version(fpath=__file__, archive_commit="96a53907", reponame="param"))
 except:
     __version__ = "0.0.0+unknown"
 
@@ -221,7 +221,7 @@ def parameterized_class(name, params, bases=Parameterized):
     supplied parameters, inheriting from the specified base(s).
     """
     if not (isinstance(bases, list) or isinstance(bases, tuple)):
-          bases=[bases]
+        bases=[bases]
     return type(name, tuple(bases), params)
 
 
@@ -924,7 +924,7 @@ class Integer(Number):
     """Numeric Parameter required to be an Integer"""
 
     def __init__(self,default=0,**params):
-       Number.__init__(self,default=default,**params)
+        Number.__init__(self,default=default,**params)
 
     def _validate(self, val):
         if callable(val): return
@@ -1014,6 +1014,10 @@ class Tuple(Parameter):
                              (self.name,len(val),self.length))
 
 
+    @classmethod
+    def deserialize(cls, value):
+        return tuple(value) # As JSON has no tuple representation
+
 
 class NumericTuple(Tuple):
     """A numeric tuple Parameter (e.g. (4.5,7.6,3)) with a fixed tuple length."""
@@ -1068,7 +1072,6 @@ def _is_abstract(class_):
         return False
 
 
-
 # CEBALERT: this should be a method of ClassSelector.
 def concrete_descendents(parentclass):
     """
@@ -1084,7 +1087,6 @@ def concrete_descendents(parentclass):
                 if not _is_abstract(c))
 
 
-
 class Composite(Parameter):
     """
     A Parameter that is a composite of a set of other attributes of the class.
@@ -1248,7 +1250,7 @@ class ObjectSelector(SelectorBase):
         to check each item instead.
         """
         if not (val in self.objects):
-           self.objects.append(val)
+            self.objects.append(val)
 
     def get_range(self):
         """
@@ -1290,6 +1292,7 @@ class Selector(ObjectSelector):
                                       check_on_set=check_on_set,
                                       allow_None=allow_None, **params)
 
+
 class ClassSelector(SelectorBase):
     """
     Parameter allowing selection of either a subclass or an instance of a given set of classes.
@@ -1306,7 +1309,6 @@ class ClassSelector(SelectorBase):
         super(ClassSelector,self).__init__(default=default,instantiate=instantiate,**params)
         self._validate(default)
 
-
     def _validate(self,val):
         """val must be None, an instance of self.class_ if self.is_instance=True or a subclass of self_class if self.is_instance=False"""
         if isinstance(self.class_, tuple):
@@ -1324,7 +1326,6 @@ class ClassSelector(SelectorBase):
                     "Parameter '%s' must be a subclass of %s, not '%s'" %
                     (val.__name__, class_name, val.__class__.__name__))
 
-
     def get_range(self):
         """
         Return the possible types for this parameter's value.
@@ -1396,7 +1397,6 @@ class List(Parameter):
                 assert isinstance(v,self.class_),repr(self.name)+": "+repr(v)+" is not an instance of " + repr(self.class_) + "."
 
 
-
 class HookList(List):
     """
     Parameter whose value is a list of callable objects.
@@ -1412,7 +1412,6 @@ class HookList(List):
             assert callable(v),repr(self.name)+": "+repr(v)+" is not callable."
 
 
-
 class Dict(ClassSelector):
     """
     Parameter whose value is a dictionary.
@@ -1431,6 +1430,15 @@ class Array(ClassSelector):
         from numpy import ndarray
         super(Array,self).__init__(ndarray, allow_None=True, default=default, **params)
 
+    @classmethod
+    def serialize(cls, value):
+        return value.tolist()
+
+    @classmethod
+    def deserialize(cls, value):
+        from numpy import asarray
+        return asarray(value)
+
 
 class DataFrame(ClassSelector):
     """
@@ -1457,7 +1465,7 @@ class DataFrame(ClassSelector):
         self.rows = rows
         self.columns = columns
         self.ordered = ordered
-        super(DataFrame,self).__init__(pdDFrame, default=default, allow_None=True, **params)
+        super(DataFrame,self).__init__(pdDFrame, default=default, **params)
         self._validate(self.default)
 
 
@@ -1502,6 +1510,15 @@ class DataFrame(ClassSelector):
         if self.rows is not None:
             self._length_bounds_check(self.rows, len(val), 'Row')
 
+    @classmethod
+    def serialize(cls, value):
+        return value.to_dict('records')
+
+    @classmethod
+    def deserialize(cls, value):
+        from pandas import DataFrame as pdDFrame
+        return pdDFrame(value)
+
 
 class Series(ClassSelector):
     """
@@ -1843,6 +1860,16 @@ class Date(Number):
 
         self._checkBounds(val)
 
+    @classmethod
+    def serialize(cls, value):
+        if not isinstance(value, (dt.datetime, dt.date)): # i.e np.datetime64
+            value = value.astype(dt.datetime)
+        return value.strftime("%Y-%m-%dT%H:%M:%S.%f")
+
+    @classmethod
+    def deserialize(cls, value):
+        return dt.datetime.strptime(value, "%Y-%m-%dT%H:%M:%S.%f")
+
 
 class CalendarDate(Number):
     """
@@ -1868,6 +1895,14 @@ class CalendarDate(Number):
 
         self._checkBounds(val)
 
+    @classmethod
+    def serialize(cls, value):
+        return value.strftime("%Y-%m-%d")
+
+    @classmethod
+    def deserialize(cls, value):
+        return dt.datetime.strptime(value, "%Y-%m-%d").date()
+
 
 class Color(Parameter):
     """
@@ -1875,7 +1910,7 @@ class Color(Parameter):
     prefix.
     """
 
-    def __init__(self, default=None, allow_None=False, **kwargs):
+    def __init__(self, default=None, **kwargs):
         super(Color, self).__init__(default=default, **kwargs)
         self._validate(default)
 
@@ -1977,7 +2012,7 @@ class DateRange(Range):
 
         start, end = val
         if not end >= start:
-           raise ValueError("DateRange '%s': end datetime %s is before start datetime %s."%(self.name,val[1],val[0]))
+            raise ValueError("DateRange '%s': end datetime %s is before start datetime %s."%(self.name,val[1],val[0]))
 
         # Calling super(DateRange, self)._check(val) would also check
         # values are numeric, which is redundant, so just call
@@ -1999,9 +2034,52 @@ class CalendarDateRange(Range):
 
         start, end = val
         if not end >= start:
-           raise ValueError("CalendarDateRange '%s': end date %s is before start date %s."%(self.name,val[1],val[0]))
+            raise ValueError("CalendarDateRange '%s': end date %s is before start date %s."%(self.name,val[1],val[0]))
 
         # Calling super(CalendarDateRange, self)._check(val) would also check
         # values are numeric, which is redundant, so just call
         # _checkBounds().
         self._checkBounds(val)
+
+
+class Event(Boolean):
+    """
+    An Event Parameter is one whose value is intimately linked to the
+    triggering of events for watchers to consume. Event has a Boolean
+    value, which when set to True triggers the associated watchers (as
+    any Parameter does) and then is automatically set back to
+    False. Conversely, if events are triggered directly via `.trigger`,
+    the value is transiently set to True (so that it's clear which of
+    many parameters being watched may have changed), then restored to
+    False when the triggering completes. An Event parameter is thus like
+    a momentary switch or pushbutton with a transient True value that
+    serves only to launch some other action (e.g. via a param.depends
+    decorator), rather than encapsulating the action itself as
+    param.Action does.
+    """
+
+    # _autotrigger_value specifies the value used to set the parameter
+    # to when the parameter is supplied to the trigger method. This
+    # value change is then what triggers the watcher callbacks.
+    __slots__ = ['_autotrigger_value', '_mode', '_autotrigger_reset_value']
+
+    def __init__(self,default=False,bounds=(0,1),**params):
+        self._autotrigger_value = True
+        self._autotrigger_reset_value = False
+        self._mode = 'set-reset'
+        super(Event, self).__init__(default=default,**params)
+
+    def _reset_event(self, obj, val):
+        val = False
+        if obj is None:
+            self.default = val
+        else:
+            obj.__dict__[self._internal_name] = val
+        self._post_setter(obj, val)
+
+    @instance_descriptor
+    def __set__(self, obj, val):
+        if self._mode in ['set-reset', 'set']:
+            super(Event, self).__set__(obj, val)
+        if self._mode in ['set-reset', 'reset']:
+            self._reset_event(obj, val)
